"""
created by P.Cizek
"""
import numpy as np
import math

import collections
import heapq

import visualization as vis

import matplotlib.pyplot as plt

class D_Star_Map:
	def __init__(self, grid, goal):
		(self.width, self.height) = np.shape(grid)
		self.grid = grid
		self.grid_rhs = np.full((self.width, self.height), np.inf)
		self.grid_g = np.full((self.width, self.height), np.inf)
		self.goal = goal
	
	###############################
	#whole grid get methods
	###############################	
	def get_grid(self):
		#get the grid
		return self.grid
	
	def get_rhs(self):
		#get rhs grid
		return self.grid_rhs
	
	def get_g(self):
		#get g grid
		return self.grid_g
	
	##############################################################
	#individual cells get and set methods 
	##############################################################
	def set_cell(self, id, value):
		(x, y) = id
		self.grid[x][y] = value
	
	def set_cell_rhs(self, id, value):
		#set rhs value
		(x, y) = id
		self.grid_rhs[x][y] = value
	
	def get_cell_rhs(self, id):
		#get rhs value
		(x, y) = id
		return self.grid_rhs[x][y]
	
	def set_cell_g(self,id, value):
		#set g value
		(x, y) = id
		self.grid_g[x][y] = value
	
	def get_cell_g(self, id):
		#get g value
		(x, y) = id
		return self.grid_g[x][y]
	
	###############################
	#map helper functions
	###############################
	def in_bounds(self, id):
		#check the boundaries of the map
		(x,y) = id
		return 0 <= x < self.width and 0 <= y < self.height
	
	def passable(self, id):
		#check the passability of the given cell
		(x,y) = id
		return self.grid[x, y] == 1
	
	def neighbors4(self, id):
		#returns coordinates of passable neighbors of the given cell in 4-neighborhood
		(x, y) = id
		results = [(x+1, y), (x, y-1), (x-1, y), (x, y+1)]
		results = filter(self.in_bounds, results)
		results = filter(self.passable, results)
		return results
	
	###############################
	#d_star helper functions
	###############################
	def calculateKey(self, id):
		#return key
		(x, y) = id
		return (np.minimum(self.grid_g[x][y], self.grid_rhs[x][y]) + self.focusing_heuristics(id)) #, np.minimum(self.grid_g[x][y], self.grid_rhs[x][y]))
	
	def focusing_heuristics(self, id):
		#focusing heuristics as manhattan distance
		(xc, yc) = id
		(xg, yg) = self.goal
		return (abs(xc - xg) + abs(yc - yg))


###############################
#priority queue class
###############################
class PriorityQueue:
		def __init__(self):
			self.elements = []
		
		def empty(self):
			return len(self.elements) == 0
		
		def put(self, item, priority):
			heapq.heappush(self.elements, (priority, item))
		
		def get(self):
			return heapq.heappop(self.elements)[1]
		
		def top(self):
			u = self.elements[0]
			return u[1]
		
		def contains(self, element):
			ret = False
			for item in self.elements:
				if element == item[1]:
					ret = True
					break
			return ret
		
		def print_elements(self):
			print(self.elements)
		
		def remove(self, element):
			i = 0
			for item in self.elements:
				if element == item[1]:
					self.elements[i] = self.elements[-1]
					self.elements.pop()
					heapq.heapify(self.elements)
					break
				i += 1
			
			print(self.elements)

###############################
#D star lite planner
###############################
def compute_shortest_path(grid_map, U, start, goal):
	#implementation of compute shortest path function

def update_vertex(grid_map, U, u, start, goal):
	#implementation of update vertex function

if __name__ == "__main__":
	#prepare map
	t = np.ones((10, 10))
	
	t[5, 5] = 0
	t[5, 4] = 0
	t[4, 5] = 0
	t[3, 5] = 0
	t[2, 5] = 0
	
	obstacles = [(5,0),(5,1),(5,2),(5,3),(5,4),(1,1),(1,2),(1,3),(1,4),(7,1),(7,2),(7,3),(7,4),(7,5),(7,6),(7,7),(7,8),(8,8),(9,8),(1,7),(2,7),(3,7),(4,7),(5,7),(6,7)]
	
	start = (0, 0)
	goal = (3, 3)
	fig, ax = plt.subplots(figsize=(10, 10), dpi=40)
	
	
	###############################
	#initialize d_star_lite
	###############################
	grid_map = D_Star_Map(t, goal)
	grid_map.set_cell_rhs(goal, 0)
	
	U = PriorityQueue()
	U.put(goal, grid_map.calculateKey(goal))
	
	#compute shortest path
	compute_shortest_path(grid_map, U, start, goal)
	
	
	###############################
	#execute and replan
	###############################
	while start != goal:
		#plot the map
		vis.plot_map(ax, t)
		vis.annotate_map(ax, grid_map.get_g(), (-0.1, -0.2))
		vis.annotate_map(ax, grid_map.get_rhs(), (-0.1, 0.2))
		
		#take one step towards goal
		neighbors = grid_map.neighbors4(start)
		start_prime = start
		for id in neighbors:
			if grid_map.get_cell_g(id) < grid_map.get_cell_g(start_prime):
				start_prime = id
		print(start_prime)
		
		if start_prime in obstacles:
			print("blocked")
			grid_map.set_cell(start_prime, 0) 
		
		if not grid_map.passable(start_prime):
			#the path is blocked
		else:
			#move towards the target
			start = start_prime
		
		#plot the robot
		vis.plot_path(ax,(start, start))
		plt.pause(0.2)
		
		