import sys
import matplotlib.pyplot as plt 
import numpy as np

def plot_map(ax, grid_map):
	#method to plot the grid map
	size = np.shape(grid_map)
	ax.cla()
	ax.imshow(np.transpose(grid_map), cmap="hot", interpolation="nearest")
	ax.set_xticks(np.arange(0.5, size[0], 1))
	ax.set_yticks(np.arange(0.5, size[1], 1))
	ax.grid()
	plt.draw()

def plot_path(ax, path):
	#method to plot the path given as list of tuples
	x_val = [x[0] for x in path]
	y_val = [x[1] for x in path]
	ax.plot(x_val, y_val)
	ax.plot(x_val, y_val, 'or')
	plt.draw()

def annotate_map(ax, values, offset):
	#method to annotate the grid map with values
	size = np.shape(values)
	for i in range(0, size[0]):
		for j in range(0, size[1]):
			ax.annotate(values[i][j], xy=(i + offset[0], j + offset[1]),color="blue")

def onclick(event):
	#helper callback method for obstacle placement
	print('click:', event.button, event.x, event.y, event.xdata, event.ydata)
	x = int(round(event.xdata))
	y = int(round(event.ydata))
	
	#add to obstacles if it is not already there
	if not (x, y) in obstacles:
		obstacles.append((x, y))
	
	#add to the grid map and plot it 
	grid_map[x, y] = 0
	plot_map(ax, grid_map)

def onbtn(event):
	print('keyevent:', event.key, event.xdata, event.ydata)
	print(obstacles)
	sys.exit(0)

if __name__ == "__main__":
	grid_map = np.ones((10,10))

	obstacles = []
	
	# initialization of figure with the size of the map and 40 pixels per one cell
	size = np.shape(grid_map)
	fig, ax = plt.subplots(figsize=(size[0], size[1]), dpi=40)

	#visualize the image
	plot_map(ax, grid_map)
	
	cid0 = fig.canvas.mpl_connect('button_press_event', onclick)
	cid1 = fig.canvas.mpl_connect('key_press_event', onbtn)
	plt.show()
